use reqwest;
use tokio;
use tokio::io::AsyncWriteExt;
use std::fs;
use std::path::{Path};

// Function to download a file from a given URL and save it
async fn download_file(url: &str, base_path: &Path) -> Result<(), Box<dyn std::error::Error>> {
    let url_obj = reqwest::Url::parse(url)?;

    let url_path_str = url_obj.path().trim_start_matches('/');

    let file_name = if url_path_str.is_empty() {
        "index.html"
    } else {
        url_path_str
    };

    let file_path = base_path.join(file_name);

    // If the file has a parent directory, create it (and any necessary parent directories)
    if let Some(parent) = file_path.parent() {
        fs::create_dir_all(parent)?;
    }

    let response = reqwest::get(url).await?;

    let mut file = tokio::fs::File::create(&file_path).await?;
    file.write_all(&response.bytes().await?).await?;

    println!("[+] Downloaded: {}", file_name);
    Ok(())
}

// Function to download all files from a list of URLs
async fn download_all(urls: Vec<&str>) -> Result<(), Box<dyn std::error::Error>> {
    let base_path = Path::new("downloaded_files");

    fs::create_dir_all(base_path)?;

    // Create a vector of asynchronous tasks to download each URL concurrently
    let tasks: Vec<_> = urls
        .iter()
        .map(|url| download_file(url, &base_path))
        .collect();

    // Concurrently await all the download tasks
    futures::future::try_join_all(tasks).await?;

    Ok(())
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    // Collect command-line arguments
    let args: Vec<String> = std::env::args().collect();

    if args.len() <= 1 {
        eprintln!("[-] You have to provide at least one URL");
        std::process::exit(1);
    }

    // Extract URLs from command-line arguments (skipping the first argument which is the program name)
    let urls: Vec<&str> = args.iter().skip(1).map(|s| s.as_str()).collect();

    // Call the download_all function to download files from the provided URLs
    download_all(urls).await?;

    Ok(())
}

